/**  Kylin security execute control
 *
 *  This file contains security  apis for
 *  Kylin security exectl function
 *
 *  Authors: jiayongfeng<jiayongfeng@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 */

#ifndef _NETFLOW_H
#define _NETFLOW_H

#include <stdint.h>
#include <time.h>
#include <netinet/in.h>


/*******************************************************************************************/

/* 网络流量的出站和入站方向 */
typedef enum {
    NETFLOW_INBOUND =  0 , // 入站网络流量
    NETFLOW_OUTBOUND,      // 出站网络流量
    NETFLOW_ALL            // 所有流量
} netflow_dir_t;

/* IP地址版本定义 */
typedef enum {
	IP_VERSION_IPV4 = 0,
	IP_VERSION_IPV6
} ip_version_t;


/* IP地址定义 */
union ip_addr {
	struct in_addr ipv4;			// IPv4地址
	struct in6_addr ipv6;			// IPv6地址
};

/* 说明： 若要表示任意IP地址时，IPv4是 0.0.0.0 ， IPv6是 ::0 */
/* IP地址范围 */
struct ip_range {
	union ip_addr start;  			// 范围起点
	union ip_addr end;				// 范围终点
};

/* 端口范围 */
struct port_range {
	uint16_t start;					// 范围起点
	uint16_t end;					// 范围终点
};

/* 网络五元组结构体定义 */
struct net_five_tuple {
	struct ip_range src_ip_range;     // 源IP地址范围，为固定值时，start,end为相同的值
	struct ip_range dst_ip_range;     // 目标IP地址范围，为固定值时，start,end为相同的值
	struct port_range src_port_range; // 源端口范围，为固定值时，start,end为相同的值
	struct port_range dst_port_range; // 目标端口范围，为固定值时，start,end为相同的值
	uint8_t protocol;        // 协议类型
	ip_version_t ip_version; // IP协议版本
};

/* 网络流量审计策略 */
struct netflow_audit_policy
{ 
	uint16_t policy_id; 		   //策略编号
	char process_name[16];         //进程名称
	struct net_five_tuple tuple;	//五元组信息
	netflow_dir_t direction; 		//网络流量方向
	int type; 					    // 策略的类型，1. 进程，2.五元组
};

/* 网络流量审计数据包信息 */
struct netflow_audit_pkt
{	
	uint16_t policy_id;          // 策略编号
	pid_t pid;       			//数据包所属进程pid
    char  process_name[16];	    //数据包所属进程名
	struct net_five_tuple tuple;	//五元组信息
    int pkt_len;     			   //数据包长度
    char payload[256]; //数据包审计负载,只截取每个包的前面256字节
	netflow_dir_t direction; 		//网络流量方向
};

//网络流量阻断日志内容
struct netflow_drop_log {
    struct net_five_tuple tuple_info; //五元组
    time_t timestamp;                 //时间戳
    pid_t pid;                        //进程PID
    char process_name[16];            //进程名
    uint16_t policy_id;               //策略编号
    netflow_dir_t direction;          //网络流量方向
};

//网络流量阻断策略
struct netflow_drop_policy {
    uint16_t policy_id;                //策略编号
    struct   net_five_tuple tuple;     //网络五元组信息
    netflow_dir_t direction;           //要管控的网络流量方向
};

//添加阻断策略
int netflow_drop_add_policy(struct netflow_drop_policy *policy);

//删除阻断策略
int netflow_drop_delete_policy(uint16_t policy_id);

//清空阻断策略
int netflow_drop_clear_policy();

//获取已经下发的所有阻断策略
struct netflow_drop_policy* netflow_drop_policies(int *count);


int netflow_drop_log_open();

int netflow_drop_log_recv(int socket_fd,void * buffer,size_t length);

int netflow_drop_log_close(int socket_fd);

/*******************************************************************************************/

// 添加基于进程或五元组的网络流量审计策略
int netflow_audit_add_policy(struct netflow_audit_policy * policy);

// 删除（单条）网络流量审计策略接口
int netflow_audit_delete_policy(uint16_t policy_id);

// 清空策略
int netflow_audit_clear_policy(void);

/* 此接口返回的一块连续的内存，里面存放着count条策略，类似数组方式取每一条策略，调用者最后释放返回的这个指针所指的内存即可 */
// 查询当前策略
struct netflow_audit_policy * netflow_audit_policies(int *count);

// 初始化 报文获取句柄
int netflow_audit_socket_open(void);

// 接收进程流量数据信息接口
size_t netflow_audit_socket_recv(int socket_fd, void *buffer, size_t length);

// 关闭进程流量数据信息句柄接口
int netflow_audit_socket_close(int socket_fd);

#endif
